@@ -3,7 +3,7 @@ import { Sequelize } from 'sequelize';
 import pg from 'pg';
 import util from 'util';
 
-import { SYNC_DIRECTIONS } from '@tamanu/constants';
+import { SYNC_DIRECTIONS, AUDIT_USERID_KEY, SESSION_CONFIG_PREFIX } from '@tamanu/constants';
 import { log } from '@tamanu/shared/services/logging';
 import { serviceContext, serviceName } from '@tamanu/shared/services/logging/context';
 
@@ -14,17 +14,28 @@ import { setupQuote } from '../utils/pgComposite';
 
 createDateTypes();
 
+export const asyncLocalStorage = new AsyncLocalStorage();
 // this allows us to use transaction callbacks without manually managing a transaction handle
 // https://sequelize.org/master/manual/transactions.html#automatically-pass-transactions-to-all-queries
 // done once for all sequelize objects. Instead of cls-hooked we use the built-in AsyncLocalStorage.
-const asyncLocalStorage = new AsyncLocalStorage();
-// eslint-disable-next-line react-hooks/rules-of-hooks
-Sequelize.useCLS({
+export const namespace = {
   bind: () => {}, // compatibility with cls-hooked, not used by sequelize
   get: (id) => asyncLocalStorage.getStore()?.get(id),
   set: (id, value) => asyncLocalStorage.getStore()?.set(id, value),
   run: (callback) => asyncLocalStorage.run(new Map(), callback),
-});
+};
+
+export const setSessionConfigInNamespace = (key, value, callback) => {
+  namespace.run(() => {
+    namespace.set(`${SESSION_CONFIG_PREFIX}${key}`, value);
+    callback()
+  });
+};
+
+export const getSessionConfigInNamespace = (key) => namespace.get(`${SESSION_CONFIG_PREFIX}${key}`);
+
+// eslint-disable-next-line react-hooks/rules-of-hooks
+Sequelize.useCLS(namespace);
 
 // this is dangerous and should only be used in test mode
 const unsafeRecreatePgDb = async ({ name, username, password, host, port }) => {
@@ -64,6 +75,7 @@ async function connectToDatabase(dbOptions) {
     pool,
     alwaysCreateConnection = true,
     loggingOverride = null, // used in tests for migration determinism
+    disableChangesAudit = false,
   } = dbOptions;
   let { name } = dbOptions;
 
@@ -108,6 +120,32 @@ async function connectToDatabase(dbOptions) {
     logging,
     pool,
   });
+
+  sequelize.setSessionVar = (key, value) =>
+    sequelize.query(`SELECT public.set_session_config($key, $value)`, {
+      bind: { key, value },
+    });
+
+  sequelize.setTransactionVar = (key, value) =>
+    sequelize.query(`SELECT public.set_session_config($key, $value, true)`, {
+      bind: { key, value },
+    });
+
+  if (!disableChangesAudit) {
+    class QueryWithAuditConfig extends sequelize.dialect.Query {
+      async run(sql, options) {
+        const userid = getSessionConfigInNamespace(AUDIT_USERID_KEY);
+        if (userid)
+          await super.run(
+            'SELECT public.set_session_config($1, $2)',
+            [AUDIT_USERID_KEY, userid],
+          );
+        return super.run(sql, options);
+      }
+    }
+    sequelize.dialect.Query = QueryWithAuditConfig;
+  }
+
   setupQuote(sequelize);
   await sequelize.authenticate();
 
@@ -176,16 +214,6 @@ export async function initDatabase(dbOptions) {
       );
     } else {
       throw new Error(`Model ${modelClass.name} has no initModel()`);
-      // modelClass.init(
-      //   {
-      //     underscored: true,
-      //     primaryKey,
-      //     sequelize,
-      //     paranoid: makeEveryModelParanoid,
-      //     hackToSkipEncounterValidation,
-      //   },
-      //   models,
-      // );
     }
   });
 
@@ -199,7 +227,7 @@ export async function initDatabase(dbOptions) {
     if (
       modelClass.syncDirection === SYNC_DIRECTIONS.DO_NOT_SYNC &&
       modelClass.usesPublicSchema &&
-      !NON_SYNCING_TABLES.includes(modelClass.tableName)
+      !NON_SYNCING_TABLES.includes(`public.${modelClass.tableName}`)
     ) {
       throw new Error(
         `Any table that does not sync should be added to the "NON_SYNCING_TABLES" list. Please check ${modelClass.tableName}`,
